# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: protobuf.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = ()


import betterproto2

_COMPILER_VERSION = "0.9.0"
betterproto2.check_compiler_version(_COMPILER_VERSION)
