syntax = "proto3";

package import_cousin_package.test.subpackage;

import "cousin.proto";

// Verify that we can import message unrelated to us

message Test {
  cousin.cousin_subpackage.CousinMessage message = 1;
}
