syntax = "proto3";

package googletypes_response_embedded;

import "google/protobuf/wrappers.proto";

// Tests that wrapped values are supported as part of output message
service Test {
  rpc getOutput(Input) returns (Output);
}

message Input {}

message Output {
  google.protobuf.DoubleValue double_value = 1;
  google.protobuf.FloatValue float_value = 2;
  google.protobuf.Int64Value int64_value = 3;
  google.protobuf.UInt64Value uint64_value = 4;
  google.protobuf.Int32Value int32_value = 5;
  google.protobuf.UInt32Value uint32_value = 6;
  google.protobuf.BoolValue bool_value = 7;
  google.protobuf.StringValue string_value = 8;
  google.protobuf.BytesValue bytes_value = 9;
}
