syntax = "proto3";

package encoding_decoding;

message Overflow32 {
  uint32 uint = 1;
  int32 int = 2;
  sint32 sint = 3;
}

message Overflow64 {
  uint64 uint = 1;
  int64 int = 2;
  sint64 sint = 3;
}
