syntax = "proto3";

package deprecated;

// Some documentation about the Test message.
message Test {
  Message message = 1 [deprecated = true];
  int32 value = 2;
  message Nested {
    int32 nested_value = 1 [deprecated = true];
  }
}

message Message {
  option deprecated = true;
  string value = 1;
}

message Empty {}

service TestService {
  rpc func(Empty) returns (Empty);
  rpc deprecated_func(Empty) returns (Empty) {
    option deprecated = true;
  }
}
