syntax = "proto3";

package import_root_sibling;

import "sibling.proto";

// Tests generated imports when a message in the root package refers to another message in the root package

message Test {
  SiblingMessage sibling = 1;
}
